local LibStub = LibStub
local AceAddon = LibStub("AceAddon-3.0")
local AceLocale = LibStub("AceLocale-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local ControlPanel = AceAddon:GetAddon("ControlPanel")
local ControlPanel_Options = AceAddon:GetAddon("ControlPanel_Options")
local L = AceLocale:GetLocale("ControlPanel_Options")
local string_format = string.format
local ipairs = ipairs

local RestartGx = RestartGx

local post_process_anti_aliasings = {"None","FXAA Low","FXAA High","CMAA"}
local resample_qualities = {L["Bilinear"],L["Bicubic"]}
local screenshot_formats = { jpg = "jpg", jpeg = "jpeg", tga = "tga"}
local MSAA_qualities = { ["0"] = L["None"], ["1,0"]=L["Color 2x / Depth 2x"],["2,0"]=L["Color 4x / Depth 4x"],
				["2,1"]=L["Color 4x / Depth 8x"],["2,2"]=L["Color 4x / Depth 16x"],
				["3,0"]=L["Color 8x / Depth 8x"],["3,1"]=L["Color 8x / Depth 16x"],["3,2"]=L["Color 8x / Depth 32x"]}
local order = 0
local function get_order()
	local temp = order
	order = order +1
	return temp
end
	
local Graphics =
{
	name = L["Graphics"],
	type = "group",
	args =
	{
		hdPlayerModels =
		{
			order = get_order(),
			name = L["Show New Character Models"],
			desc = L["Check this to show new higher definition character models. (64-bit OS recommended)"],
			type = "toggle",
			set = function(info,val)
				ControlPanel:SetCVar("hdPlayerModels",val)
			end,
			get = function(info)
				return ControlPanel:GetCVarBool("hdPlayerModels")
			end,
			width = "full",
		},
		useUiScale =
		{
			order = get_order(),
			name = L["Use UI Scale"],
			desc = L["Check to use UI Scale Slider, uncheck to use the system default scale."],
			type = "toggle",
			set = function(info,val)
				ControlPanel:SetCVar("useUiScale",val)
			end,
			get = function(info)
				return ControlPanel:GetCVarBool("useUiScale")
			end,
		},
		uiScale =
		{
			order = get_order(),
			name = L["UI Scale"],
			desc = L["Changes the size of the game's user interface"],
			type = "range",
			min = 0,
			max = 10,
			set = function(info,val)
				ControlPanel:SetCVar("uiScale",val)
			end,
			get = function(info)
				return ControlPanel:GetCVarNumber("uiScale")
			end,
		},
		physicsLevel = 
		{
			order = get_order(),
			name = L["Physics Interactions"],
			type = "range",
			min = 0,
			max = 2,
			step = 1,
			set = function(info,val)
				ControlPanel:SetCVar("physicsLevel",val)
			end,
			get = function(info)
				return ControlPanel:GetCVarNumber("physicsLevel")
			end,
		},
		FPS =
		{
			order = get_order(),
			name = L["FPS"],
			type = "group",
			args =
			{
				maxFPS = 
				{
					order = get_order(),
					name = L["Max Foreground FPS"],
					desc = L["Maximum framerate while running in the foreground\n\n(0 represents no limits)"],
					type = "range",
					min = 0,
					max = 200,
					set = function(info,val)
						ControlPanel:SetCVar("maxFPS",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("maxFPS")
					end,
				},
				maxFPSBk = 
				{
					order = get_order(),
					name = L["Max Background FPS"],
					desc = L["Maximum framerate while running in the background\n\n(0 represents no limits)"],
					type = "range",
					min = 0,
					max = 200,
					set = function(info,val)
						ControlPanel:SetCVar("maxFPSBk",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("maxFPSBk")
					end,
				},
			}
		},
		Anti_Aliasing =
		{
			order = get_order(),
			name = L["Anti-Aliasing"],
			desc = L["Anti-Aliasing is a technique to smooth out jaggy edges, especially at low resolutions, but can result in a slightly blurred image. Enabling Anti-Aliasing will reduce performance."],
			type = "group",
			args =
			{
				ffxAntiAliasingMode =
				{
					order = get_order(),
					name = L["Post-Process AA"],
					type = "select",
					values = post_process_anti_aliasings,
					set = function(info,val)
						ControlPanel:SetCVar("ffxAntiAliasingMode",val-1)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("ffxAntiAliasingMode")+1
					end,
					width = "full",
				},
				MSAA =
				{
					order = get_order(),
					name = "MSAA",
					desc = L["Multisample anti-aliasing"],
					type = "group",
					args =
					{
						MSAAQuality =
						{
							order = get_order(),
							name = "MSAA",
							desc = L["Sets the Multisample Anti-Aliasing mode. Both the number of color and depth samples can be specified."],
							type = "select",
							values = MSAA_qualities,
							set = function(info,val)
								ControlPanel:SetCVar("MSAAQuality",val)
							end,
							get = function(info)
								return ControlPanel:GetCVar("MSAAQuality")
							end,
						},
						MSAAAlphaTest =
						{
							order = get_order(),
							name = L["Alpha-Test"],
							desc = L["Evaluates alpha-test operation at sample frequency rather than pixel frequency when MSAA is enabled."],
							type = "toggle",
							set = function(info,val)
								ControlPanel:SetCVar("MSAAAlphaTest",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarBool("MSAAAlphaTest")
							end,
						}
					}
				},
				SSAA =
				{
					order = get_order(),
					name = "SSAA",
					desc = L["Supersampling Anti-aliasing"],
					type = "group",
					args =
					{
						RenderScale =
						{
							order = get_order(),
							name = L["Render Scale"],
							desc = L["Multiplier on display resolution for rendering the 3D World."],
							type = "range",
							min = 0,
							max = 2,
							set = function(info,val)
								ControlPanel:SetCVar("RenderScale",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("RenderScale")
							end,
						},
						ResampleQuality =
						{
							order = get_order(),
							name = L["Resample Quality"],
							desc = L["When using a Render Scale that is not 100%, this adjusts the quality of the filter to display resolution."],
							type = "select",
							values = resample_qualities,
							set = function(info,val)
								ControlPanel:SetCVar("ResampleQuality",val-1)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("ResampleQuality")+1
							end,
						}
					}
				},
			}
		},
		BasicSettings = 
		{
			order = get_order(),
			name = L["Basic Settings"],
			type = "group",
			args = 
			{
				graphicsQuality = 
				{
					order = get_order(),
					name = L["Graphics Quality"],
					type = "range",
					min = 1,
					max = 10,
					step = 1,
					set = function(info,val)
						ControlPanel:SetCVar("graphicsQuality",val)
						ControlPanel_Options:set_graphics_quality(val)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("graphicsQuality")
					end,
					width = "full",
				},
				Textures =
				{
					order = get_order(),
					name = L["Textures"],
					type = "group",
					args =
					{
						graphicsTextureResolution = 
						{
							order = get_order(),
							name = L["Texture Resolution"],
							--desc = L["Controls the level of terrain blending. (Lower is better.) Increasing this may slightly improve performance."],
							type = "range",
							min = 1,
							max = 3,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsTextureResolution",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsTextureResolution")
							end,
						},
						graphicsTextureFiltering = 
						{
							order = get_order(),
							name = L["Texture Filtering"],
							--desc = L["Increases texture sharpness, particularly for textures view at an angle. Decrease to improve performance.\n\nBilinear(0)\nTrilinear(1)\n2x Anisotropic(2)\n4x Anisotropic(3)\n8x Anisotropic(4)\n16x Anisotropic(5)\n"],
							type = "range",
							min = 1,
							max = 6,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsTextureFiltering",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsTextureFiltering")
							end,
						},
						graphicsProjectedTextures = 
						{
							order = get_order(),
							name = L["Projected Textures"],
							desc = L["Enables the projecting of textures to the environment. Disabling this may greatly improve performance."],
							type = "toggle",
							set = function(info,val)
								if val == true then
									ControlPanel:SetCVar("graphicsProjectedTextures",2)
								else
									ControlPanel:SetCVar("graphicsProjectedTextures",1)
								end
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsProjectedTextures")==2
							end,
						}
					}
				},
				Environment =
				{
					order = get_order(),
					name = L["Environment"],
					type = "group",
					args = 
					{
						graphicsViewDistance = 
						{
							order = get_order(),
							name = L["View Distance"],
							desc = L["View distance controls how far you can see. Larger view distances require more memory and a faster processor."],
							type = "range",
							min = 1,
							max = 10,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsViewDistance",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsViewDistance")
							end,
						},
						graphicsEnvironmentDetail = 
						{
							order = get_order(),
							name = L["Environment Detail"],
							desc = L["Controls how far you can see objects. Decrease to improve performance."],
							type = "range",
							min = 1,
							max = 10,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsEnvironmentDetail",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsEnvironmentDetail")
							end,
						},
						graphicsGroundClutter = 
						{
							order = get_order(),
							name = L["Ground Clutter"],
							type = "range",
							min = 1,
							max = 10,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsGroundClutter",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsGroundClutter")
							end,
						},
					}
				},
				Effects =
				{
					order = get_order(),
					name = L["Effects"],
					type = "group",
					args = 
					{
						graphicsShadowQuality = 
						{
							order = get_order(),
							name = L["Shadow Quality"],
							desc = L["Controls both the method and quality of shadows. Decreasing this may greatly improve performance."],
							type = "range",
							min = 1,
							max = 6,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsShadowQuality",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsShadowQuality")
							end,
						},
						graphicsLiquidDetail = 
						{
							order = get_order(),
							name = L["Liquid Detail"],
							desc = L["Controls the rendering quality of liquid. Decreasing this may improve performance.\n\nLow(0): Animated Liquid textures, texture based ripples and no reflection.\n\nFair(1): Normalmap liquid textures, texture based ripples and sky reflection.\n\nGood(2): Normalmap liquid textures, procedural ripples and screen-based reflection.\n\nUltra(3): Normalmap liquid textures, procedural ripples and full reflection."],
							type = "range",
							min = 1,
							max = 4,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsLiquidDetail",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsLiquidDetail")
							end,
						},
						graphicsSunshafts = 
						{
							order = get_order(),
							name = L["Sunshafts"],
							desc = L["Controls the method and quality used to display sunshafts and glare effects. Disabling this setting may improve performance.\n\nDisabled(0): Traditional sunshafts and glave.\n\nLow(1): Depth-based sunshafts and glare.\n\nHigh(2): Depth-based sunshafts and glare with improved sampling to reduce anti-aliasing."],
							type = "range",
							min = 1,
							max = 3,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsSunshafts",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsSunshafts")
							end,
						},
						graphicsParticleDensity = 
						{
							order = get_order(),
							name = L["Particle Density"],
							desc = L["Controls the number of particles used in effects caused by spells, fires, etc. Decrease to improve performance."],
							type = "range",
							min = 1,
							max = 4,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsParticleDensity",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsParticleDensity")
							end,
						},
						graphicsSSAO = 
						{
							order = get_order(),
							name = L["SSAO"],
							desc = L["Controls the rendering quality of the advanced lighting effects. Decreasing this may greatly improve performance."],
							type = "range",
							min = 1,
							max = 4,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsSSAO",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsSSAO")
							end,
						},
						graphicsDepthEffects = 
						{
							order = get_order(),
							name = L["Depth Effects"],
							type = "range",
							min = 1,
							max = 4,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsDepthEffects",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsDepthEffects")
							end,
						},
						graphicsLightingQuality = 
						{
							order = get_order(),
							name = L["Lightning Quality"],
							desc = L["Controls both the method and quality of lighting. Decreasing this may improve performance."],
							type = "range",
							min = 1,
							max = 3,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsLightingQuality",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("graphicsLightingQuality")
							end,
						},
						graphicsOutlineMode = 
						{
							order = get_order(),
							name = L["Outline Mode"],
							desc = L["Controls whether the selection outline effect is allowed. See the interface Display panel for more options."],
							type = "range",
							min = 1,
							max = 3,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("graphicsOutlineMode",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarBool("graphicsOutlineMode")
							end,
						}
					}
				}
			}
		},
		
		RaidAndBattlegroundSettings = 
		{
			order = get_order(),
			name = L["Raid And Battleground Settings"],
			type = "group",
			args = 
			{
				raidGraphicsQuality = 
				{
					order = get_order(),
					name = L["Graphics Quality"],
					type = "range",
					min = 1,
					max = 10,
					step = 1,
					set = function(info,val)
						ControlPanel:SetCVar("RAIDgraphicsQuality",val)
						ControlPanel_Options:set_raid_graphics_quality(val)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("RAIDgraphicsQuality")
					end,
					width = "full",
				},
				RAIDsettingsEnabled =
				{
					order = get_order(),
					name = L["Enable"],
					desc = L["Check this option to enable alternate graphic setting for Raids and Battlegrounds"],
					type = "toggle",
					set = function(info,val)
						ControlPanel:SetCVar("RAIDsettingsEnabled",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarBool("RAIDsettingsEnabled")
					end,
					width = "full",
				},
				Textures =
				{
					order = get_order(),
					name = L["Textures"],
					type = "group",
					args =
					{
						raidGraphicsTextureResolution = 
						{
							order = get_order(),
							name = L["Texture Resolution"],
							--desc = L["Controls the level of terrain blending. (Lower is better.) Increasing this may slightly improve performance."],
							type = "range",
							min = 1,
							max = 3,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsTextureResolution",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsTextureResolution")
							end,
						},
						raidGraphicsTextureFiltering = 
						{
							order = get_order(),
							name = L["Texture Filtering"],
							--desc = L["Increases texture sharpness, particularly for textures view at an angle. Decrease to improve performance.\n\nBilinear(0)\nTrilinear(1)\n2x Anisotropic(2)\n4x Anisotropic(3)\n8x Anisotropic(4)\n16x Anisotropic(5)\n"],
							type = "range",
							min = 1,
							max = 6,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsTextureFiltering",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsTextureFiltering")
							end,
						},
						raidGraphicsProjectedTextures = 
						{
							order = get_order(),
							name = L["Projected Textures"],
							desc = L["Enables the projecting of textures to the environment. Disabling this may greatly improve performance."],
							type = "toggle",
							set = function(info,val)
								if val == true then
									ControlPanel:SetCVar("raidGraphicsProjectedTextures",2)
								else
									ControlPanel:SetCVar("raidGraphicsProjectedTextures",1)
								end
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsProjectedTextures")==2
							end,
						}
					}
				},
				Environment =
				{
					order = get_order(),
					name = L["Environment"],
					type = "group",
					args = 
					{
						raidGraphicsViewDistance = 
						{
							order = get_order(),
							name = L["View Distance"],
							desc = L["View distance controls how far you can see. Larger view distances require more memory and a faster processor."],
							type = "range",
							min = 1,
							max = 10,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsViewDistance",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsViewDistance")
							end,
						},
						raidGraphicsEnvironmentDetail = 
						{
							order = get_order(),
							name = L["Environment Detail"],
							desc = L["Controls how far you can see objects. Decrease to improve performance."],
							type = "range",
							min = 1,
							max = 10,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsEnvironmentDetail",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsEnvironmentDetail")
							end,
						},
						raidGraphicsGroundClutter = 
						{
							order = get_order(),
							name = L["Ground Clutter"],
							type = "range",
							min = 1,
							max = 10,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsGroundClutter",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsGroundClutter")
							end,
						},
					}
				},
				Effects =
				{
					order = get_order(),
					name = L["Effects"],
					type = "group",
					args = 
					{
						raidGraphicsShadowQuality = 
						{
							order = get_order(),
							name = L["Shadow Quality"],
							desc = L["Controls both the method and quality of shadows. Decreasing this may greatly improve performance."],
							type = "range",
							min = 1,
							max = 6,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsShadowQuality",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsShadowQuality")
							end,
						},
						raidGraphicsLiquidDetail = 
						{
							order = get_order(),
							name = L["Liquid Detail"],
							desc = L["Controls the rendering quality of liquid. Decreasing this may improve performance.\n\nLow(0): Animated Liquid textures, texture based ripples and no reflection.\n\nFair(1): Normalmap liquid textures, texture based ripples and sky reflection.\n\nGood(2): Normalmap liquid textures, procedural ripples and screen-based reflection.\n\nUltra(3): Normalmap liquid textures, procedural ripples and full reflection."],
							type = "range",
							min = 1,
							max = 4,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsLiquidDetail",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsLiquidDetail")
							end,
						},
						raidGraphicsSunshafts = 
						{
							order = get_order(),
							name = L["Sunshafts"],
							desc = L["Controls the method and quality used to display sunshafts and glare effects. Disabling this setting may improve performance.\n\nDisabled(0): Traditional sunshafts and glave.\n\nLow(1): Depth-based sunshafts and glare.\n\nHigh(2): Depth-based sunshafts and glare with improved sampling to reduce anti-aliasing."],
							type = "range",
							min = 1,
							max = 3,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsSunshafts",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsSunshafts")
							end,
						},
						raidGraphicsParticleDensity = 
						{
							order = get_order(),
							name = L["Particle Density"],
							desc = L["Controls the number of particles used in effects caused by spells, fires, etc. Decrease to improve performance."],
							type = "range",
							min = 1,
							max = 4,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsParticleDensity",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsParticleDensity")
							end,
						},
						raidGraphicsSSAO = 
						{
							order = get_order(),
							name = L["SSAO"],
							desc = L["Controls the rendering quality of the advanced lighting effects. Decreasing this may greatly improve performance."],
							type = "range",
							min = 1,
							max = 4,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsSSAO",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsSSAO")
							end,
						},
						raidGraphicsDepthEffects = 
						{
							order = get_order(),
							name = L["Depth Effects"],
							type = "range",
							min = 1,
							max = 4,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsDepthEffects",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsDepthEffects")
							end,
						},
						raidGraphicsLightingQuality = 
						{
							order = get_order(),
							name = L["Lightning Quality"],
							desc = L["Controls both the method and quality of lighting. Decreasing this may improve performance."],
							type = "range",
							min = 1,
							max = 3,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsLightingQuality",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("raidGraphicsLightingQuality")
							end,
						},
						raidGraphicsOutlineMode = 
						{
							order = get_order(),
							name = L["Outline Mode"],
							desc = L["Controls whether the selection outline effect is allowed. See the interface Display panel for more options."],
							type = "range",
							min = 1,
							max = 3,
							step = 1,
							set = function(info,val)
								ControlPanel:SetCVar("raidGraphicsOutlineMode",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarBool("raidGraphicsOutlineMode")
							end,
						}
					}
				}
			}
		},
		
		Screenshot =
		{
			order = get_order(),
			name = L["Screenshot"],
			type = "group",
			args = 
			{
				screenshotFormat =
				{
					order = get_order(),
					name = L["Format"],
					type = "select",
					values = screenshot_formats,
					set = function(info,val)
						ControlPanel:SetCVar("screenshotFormat",val)
					end,
					get = function(info)
						return ControlPanel:GetCVar("screenshotFormat")
					end
				},
				screenshotQuality =
				{
					order = get_order(),
					name = L["Quality"],
					desc = L["Note this option only affects JPEG screenshots (TGA screenshots always use lossless quality)."],
					type = "range",
					min = 1,
					max =10,
					step = 1,
					set = function(info,val)
						ControlPanel:SetCVar("screenshotQuality",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("screenshotQuality")
					end,
				},
			}
		}
	}
}

ControlPanel_Options:push("Graphics",Graphics)